@@ -5,7 +5,6 @@ import functools
 import importlib
 import os
 import sys
-import warnings
 
 import attr
 import py
@@ -15,13 +14,14 @@ from _pytest import nodes
 from _pytest.config import directory_arg
 from _pytest.config import hookimpl
 from _pytest.config import UsageError
-from _pytest.deprecated import PYTEST_CONFIG_GLOBAL
 from _pytest.outcomes import exit
 from _pytest.runner import collect_one_node
 
 
 class ExitCode(enum.IntEnum):
     """
+    .. versionadded:: 5.0
+
     Encodes the valid exit codes by pytest.
 
     Currently users and plugins may supply other exit codes as well.
@@ -179,26 +179,6 @@ def pytest_addoption(parser):
     )
 
 
-class _ConfigDeprecated:
-    def __init__(self, config):
-        self.__dict__["_config"] = config
-
-    def __getattr__(self, attr):
-        warnings.warn(PYTEST_CONFIG_GLOBAL, stacklevel=2)
-        return getattr(self._config, attr)
-
-    def __setattr__(self, attr, val):
-        warnings.warn(PYTEST_CONFIG_GLOBAL, stacklevel=2)
-        return setattr(self._config, attr, val)
-
-    def __repr__(self):
-        return "{}({!r})".format(type(self).__name__, self._config)
-
-
-def pytest_configure(config):
-    __import__("pytest").config = _ConfigDeprecated(config)  # compatibility
-
-
 def wrap_session(config, doit):
     """Skeleton command line program"""
     session = Session(config)
@@ -449,7 +429,7 @@ class Session(nodes.FSCollector):
             # one or more conftests are not in use at this fspath
             proxy = FSHookProxy(fspath, pm, remove_mods)
         else:
-            # all plugis are active for this fspath
+            # all plugins are active for this fspath
             proxy = self.config.hook
         return proxy
 
@@ -584,7 +564,13 @@ class Session(nodes.FSCollector):
             # Module itself, so just use that. If this special case isn't taken, then all
             # the files in the package will be yielded.
             if argpath.basename == "__init__.py":
-                yield next(m[0].collect())
+                try:
+                    yield next(m[0].collect())
+                except StopIteration:
+                    # The package collects nothing with only an __init__.py
+                    # file in it, which gets ignored by the default
+                    # "python_files" option.
+                    pass
                 return
             yield from m
 
